# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimList,
    HprimMultiple,
    HprimString,
    HprimDate,
    HprimForeign,
)


class FacslinkSegmentH(HeaderSegment):
    sending_application = HprimString(5)
    version = HprimString(11)
    processing_id = HprimString(12, default="P")
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class FacslinkSegmentP(Segment):
    patient_id = HprimString(3)
    last_name = HprimString((6, 1))
    first_name = HprimString((6, 2))
    middle_name = HprimString((6, 3))
    birthdate = HprimDate(8, fmt_date="%Y%m%d")
    sex = HprimString(9)
    ordering_physician_code = HprimString((14, 1))
    ordering_physician_last_name = HprimString((14, 2))
    ordering_physician_first_name = HprimString((14, 3))
    practice_field2 = HprimString(25)
    location_facility = HprimString((26, 1))
    location_ward = HprimString((26, 2))
    location_room = HprimString((26, 3))
    location_bed = HprimString((26, 4))

    orders = HprimForeign("O", "patient")


class FacslinkSegmentO(Segment):
    specimen_id = HprimString((3, 1))
    tests = HprimList(5, basefield=HprimMultiple(
        0,
        code=HprimString(4),
        dilution=HprimString(5)
    ))
    priority = HprimString(6)
    collection_date = HprimDate(8)
    action_code = HprimString(12)
    specimen_type = HprimString((16, 1))
    user_field = HprimString((19, 1))
    destination_connection = HprimString(25)

    results = HprimForeign("R", "order")


class FacslinkSegmentR(Segment):
    test_code = HprimString((3, 4))
    result = HprimString(4)
    # F: Final
    # P: Preliminary
    status = HprimString(9)
    operator_id = HprimString((11, 1))
    test_specimen = HprimString((11, 2))
    date = HprimDate(13)
    instrument_id = HprimString(14)


class FacslinkSegmentC(Segment):
    source = HprimString(3)
    text = HprimString(4)
    typ = HprimString(5)


class FacslinkSegmentL(Segment):
    flag = HprimString(3, default="N")


class Driver(AstmMessageV2):
    SPEC = {
        "code": "facslink",
        "label": "Facslink/Facsvia ASTM BD"
    }
    SEGMENT_CLASS_MAP = {
        "H": FacslinkSegmentH,
        "P": FacslinkSegmentP,
        "O": FacslinkSegmentO,
        "R": FacslinkSegmentR,
        "C": FacslinkSegmentC,
        "L": FacslinkSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    OPTIONS = [
        AstmMessageV2.BooleanOption(
            "facslink_exclure_preliminaire",
            label="Ignorer les résultats préliminaires (flag \"P\" dans le champ R.9)"
        )
    ]

    def __init__(self, facslink_exclure_preliminaire=False, **kwargs):
        self.facslink_exclure_preliminaire = facslink_exclure_preliminaire
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = FacslinkSegmentH(type_segment="H", **kwargs)
        message.end = FacslinkSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        if worklist.context != "order":
            raise NotImplementedError("context %s" % (worklist.context,))

        msg = self.create(date=context["now"])
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            priority={"A": "S", "S": "S", worklist.ELSE: "R"},
            action={"del": "C", "add": "A", "new": "N"}
        ):
            physician = [p for p in patient.physicians if p.type in ('prescriber', 'attending')]
            seg_p = msg.patients.add(
                patient_id=patient.patient_id,
                last_name=patient.last_name,
                first_name=patient.first_name,
                birthdate=patient.birthdate,
                sex=patient.sex,
                ordering_physician_code=physician[0].code if physician else "",
                ordering_physician_last_name=physician[0].last_name if physician else "",
                ordering_physician_first_name=physician[0].first_name if physician else "",
                location_facility=patient.location
            )
            for sample, action, priority in patient.samples:
                seg_o = seg_p.orders.add(
                    specimen_id=sample.sample_id,
                    collection_date=sample.collection_date,
                    priority=priority,
                    action_code=action,
                    specimen_type=sample.specimen
                )
                for order in sample.orders:
                    t = seg_o.tests.add()
                    t.code = order["code"]
                    t.dilution = order["dilution"]
        return msg

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                if seg_o.action_code == "Q":
                    sample = worklist.get_or_create(
                        sample_id=seg_o.specimen_id,
                        control_lot=seg_o.specimen_id,
                        type="control"
                    )
                else:
                    sample = worklist.get_or_create(
                        sample_id=seg_o.specimen_id,
                        type="patient"
                    )
                for seg_r in seg_o.results:
                    if self.facslink_exclure_preliminaire and seg_r.status == "P":
                        continue
                    comments = []
                    for seg_c in seg_r.commentaires:
                        if seg_c.text:
                            comments.append(seg_c.text)
                    sample.results.add(
                        code=seg_r.test_code,
                        value=seg_r.result,
                        comments=comments,
                        date=seg_r.date if seg_r.date else message.date,
                        operator=seg_r.operator_id,
                        instrument=seg_r.instrument_id
                    )
        return worklist
