# -*- coding: utf8 -*- from __future__ import unicode_literals

from driverng import MessageV2, Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimString,
    HprimList,
    HprimDate,
    HprimForeign,
    HprimMultiple,
)


class PianoSegmentH(HeaderSegment):

    # les champs ci-dessous ne doivent pas etre transmis lors de la demande
    # code emetteur
    emetteur = HprimString(5)
    # date emission du resultat
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class PianoSegmentP(Segment):

    # Practice Assigned Patient ID
    no_patient = HprimString(3, max_length=14)

    # nom patient
    nom = HprimString((6, 1), max_length=16)

    # prenom patient
    prenom = HprimString((6, 2), max_length=16)
    date_naissance = HprimDate(8, fmt_date="%Y%m%d")
    # M/F/U
    sexe = HprimString(9)

    orders = HprimForeign("O", "patient")


class PianoSegmentO(Segment):

    # Specimen ID
    tube = HprimString((3, 1), max_length=16)

    # Liste des codes tests
    # ^^^Test^Request.DilutionCoefficient
    tests = HprimList(5, basefield=HprimMultiple(
        0,
        code=HprimString(4),
        coeff_dilution=HprimString(5),
    ))
    # Q: QC
    action = HprimString(12)
    # type de specimen d'echantillon Sp1, Sp2 ou Control
    specimen_type = HprimString(16)

    results = HprimForeign("R", "order")


class PianoSegmentQ(Segment):
    # Le segment Q n'est pas clairement décrit dans
    # la doc, mais il y a des exemples

    tube = HprimString((3, 2))

    request = HprimString(5, default="ALL")


class PianoSegmentR(Segment):

    # code test
    # ^^^Test
    test = HprimString((3, 4), max_length=3)

    # le resultat
    valeur = HprimString(4)

    # unite
    unite = HprimString(5, max_length=6)

    #  reference ranges
    ref_ranges = HprimString(6)

    #  result abnormal flags
    flag = HprimString(7, max_length=2)

    # statut du resultat
    # F: final (manual validation)
    # P: Pending (automated transfer)
    # X: Rejected
    # I: Not measured
    # Attention ce champ n'est pas systèmatiquement envoyé
    status = HprimString(9)

    # code operator
    operator = HprimString(11, max_length=32)

    # date du resultat ou du QC
    date = HprimDate(13)


class PianoSegmentL(Segment):
    # Code de terminaison
    # N: Terminaison normale
    code = HprimString(3)


class AstmPiano(AstmMessageV2):
    SEGMENT_CLASS_MAP = {
        "H": PianoSegmentH,
        "P": PianoSegmentP,
        "Q": PianoSegmentQ,
        "O": PianoSegmentO,
        "R": PianoSegmentR,
        "L": PianoSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    def create(self, **kwargs):
        message = PianoSegmentH(type_segment="H", **kwargs)
        message.end = PianoSegmentL(type_segment="L")
        return message


class MultipleAstmPiano(list):

    def __init__(self):
        self.parser = AstmPiano()
        super(MultipleAstmPiano, self).__init__()

    def add(self):
        msg = self.parser.create()
        self.append(msg)
        return msg

    @staticmethod
    def parse(data):

        message = MultipleAstmPiano()
        block = []

        for line in data.splitlines():
            if line[0] == "L" and block:
                block.append(line)
                msg = message.parser.parse("\r\n".join(block))
                message.append(msg)
                block = []
            else:
                block.append(line)

        assert block == [], "Unparsed data {}".format(block,)

        return message

    @staticmethod
    def to_string(message):
        return "\r\n".join([message.parser.to_string(msg) for msg in message])


class Driver(MessageV2):
    SPEC = {
        "code": "piano",
        "label": "Piano",
    }

    def create(self):
        return MultipleAstmPiano()

    def get_message_type(self, message):
        for msg in message:
            if msg.queries:
                # TODO: Que faire en cas de QUERY + ORDER ?
                return self.QUERY

        return self.ORDER

    def to_string(self, message):
        return MultipleAstmPiano.to_string(message)

    def parse(self, data):
        return MultipleAstmPiano.parse(data)

    def encode_worklist(self, worklist, context):
        message = self.create()
        msg = message.add()
        for patient in worklist.get_astm_worklist(
            unique_together=["action"],
            action={"del": worklist.IGNORE, worklist.ELSE: ""},
        ):
            seg_p = msg.patients.add(
                no_patient=patient.patient_id,
                nom=patient.last_name,
                prenom=patient.first_name,
                date_naissance=patient.birthdate,
                sexe=patient.sex,
            )
            for sample, action in patient.samples:
                seg_o = seg_p.orders.add(
                    tube=sample.sample_id,
                )
                for order in sample.orders:
                    # TODO: À migrer dans MCA
                    for code_demande in order.code.split(";"):
                        if code_demande and code_demande not in [
                            t.code for t in seg_o.tests
                        ]:
                            t = seg_o.tests.add()
                            t.code = code_demande

        if msg.patients or context["query"]:
            return message

    def decode_message(self, message):
        if message[0].queries:
            worklist = Worklist(context="query")
            for msg in message:
                for seg_q in msg.queries:
                    worklist.get_or_create(
                        sample_id=seg_q.tube,
                        type='patient',
                    )
            return worklist

        worklist = Worklist(context="result")
        for msg in message:
            for seg_p in msg.patients:
                for seg_o in seg_p.orders:

                    if seg_o.specimen_type == "Control" or seg_o.action == "Q":
                        sample = worklist.get_or_create(
                            type='control',
                            sample_id=seg_o.tube,
                            control_lot=seg_o.tube,
                        )
                    else:
                        sample = worklist.get_or_create(
                            type='patient',
                            sample_id=seg_o.tube,
                        )

                    for seg_r in seg_o.results:

                        if seg_r.status not in ("F", "P", ""):
                            continue

                        if seg_r.flag:
                            alarmes = [seg_r.flag]
                        else:
                            alarmes = []

                        sample.results.add(
                            code=seg_r.test,
                            value=seg_r.valeur,
                            flags=alarmes,
                            date=seg_r.date,
                            instrument=msg.emetteur,
                            operator=seg_r.operator,
                        )
        return worklist

    def enregistrer(self, data, message=None):
        if data:
            lines = data.splitlines()
            if len(lines) == 3 and lines[1] == "Q|1|^ALL||ALL":
                return False
        return True
